<?php
/**
 * @file
 * Rules integration for the Commerce Event Ticket PDF module.
 */

/**
 * Implements hook_rules_action_info().
 */
function commerce_event_ticket_pdf_rules_action_info() {
  $defaults = array(
    'group' => t('Commerce Event Ticket'),
    'module' => 'commerce_event_ticket_pdf',
    'access callback' => 'commerce_event_ticket_rules_config_access',
  );
  $actions = array();
  $actions['commerce_event_ticket_pdf_mail'] = $defaults + array(
    'label' => t("Send an order's tickets by e-mail as attachments"),
    'base' => 'rules_action_commerce_event_ticket_pdf_mail',
    'parameter' => array(
      'commerce_order' => array(
        'type' => 'commerce_order',
        'label' => t('The order'),
      ),
      'send_without_tickets' => array(
        'type' => 'boolean',
        'label' => t('Send the e-mail even if the order has no tickets'),
        'description' => t("If enabled, this e-mail will be sent regardless of whether the order has tickets. This could be useful for sending a general-purpose order confirmation e-mail."),
        'optional' => TRUE,
        'default value' => FALSE,
      ),
      'to' => array(
        'type' => 'text',
        'label' => t('To'),
        'description' => t("The mail's recipient address. The formatting of this string must comply with RFC 2822."),
        'default value' => '[commerce-order:owner:mail]',
      ),
      'from_name' => array(
        'type' => 'text',
        'label' => t('Sender name'),
        'description' => t("The sender's name. Leave it empty to use the site-wide configured name."),
        'optional' => TRUE,
      ),
      'from_mail' => array(
        'type' => 'text',
        'label' => t('Sender e-mail address'),
        'description' => t("The sender's address. Leave it empty to use the site-wide configured address."),
        'optional' => TRUE,
      ),
      'subject' => array(
        'type' => 'text',
        'label' => t('Subject'),
        'description' => t("The mail's subject."),
        'translatable' => TRUE,
        'default value' => t('Your tickets for order [commerce-order:order-number]'),
      ),
      'body' => array(
        'type' => 'text',
        'label' => t('Body'),
        'description' => t("The mail's message HTML body."),
        'sanitize' => TRUE,
        'optional' => TRUE,
        'translatable' => TRUE,
      ),
      'plaintext' => array(
        'type' => 'text',
        'label' => t('Plain text body'),
        'description' => t("The mail's message plaintext body."),
        'optional' => TRUE,
        'translatable' => TRUE,
      ),
    ),
  );
  return $actions;
}

/**
 * Rules action: send an e-mail containing an order's tickets as attachments.
 *
 * This mimics Mime Mail's Rules action, rules_action_mimemail().
 *
 * First the order is examined to check whether it has any tickets in it (if
 * not, the function will stop). Tickets and PDF files (for each line item) are
 * created if necessary. An e-mail is sent with the provided parameters
 * (subject, body, to, etc.), and all of the line items' PDF files are attached
 * automatically.
 *
 * Note: if you are trying to send e-mails using the Mandrill module for
 * delivery, at the time of writing (2014-03-12) you will need a patch to
 * Mandrill available in this issue:
 * https://drupal.org/node/2152699
 */
function rules_action_commerce_event_ticket_pdf_mail($order, $send_without_tickets = FALSE, $to, $from_name = NULL, $from_mail = NULL, $subject, $body = NULL, $plaintext = NULL) {
  // Create the tickets for the order, if they don't already exist.
  $tickets = commerce_event_ticket_create_from_order($order);
  if (!$tickets && !$send_without_tickets) {
    return;
  }

  // Create the PDF files for each line item, if they haven't already been
  // created, and format them as attachments for the Mime Mail module.
  $attachments = array();
  $products = array();
  if ($tickets) {
    $tickets_by_line_item = array();
    foreach ($tickets as $ticket) {
      if (commerce_event_ticket_is_valid($ticket)) {
        $tickets_by_line_item[$ticket->line_item_id][$ticket->ticket_id] = $ticket;
      }
    }
    $products = array();
    $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
    foreach ($order_wrapper->commerce_line_items as $line_item_wrapper) {
      $line_item = $line_item_wrapper->value();
      if (!isset($tickets_by_line_item[$line_item->line_item_id])) {
        continue;
      }
      $line_item_tickets = $tickets_by_line_item[$line_item->line_item_id];
      $file = commerce_event_ticket_pdf_create($line_item, 'commerce_line_item', $line_item_tickets);
      if (!$file) {
        continue;
      }
      $attachment['uri'] = $file->uri;
      $attachment['filepath'] = $file->uri;
      // Create a filename based on the product title.
      $product = $line_item_wrapper->commerce_product->value();
      // Sanitize the title so it can only contain letters, numbers,
      // hyphens, and underscores.
      $sanitized = preg_replace('/[^0-9a-z\-]+/i', '_', trim($product->title));
      $attachment['filename'] = $sanitized . '.pdf';
      $products[$product->product_id] = $product->title;
      $attachments[] = $attachment;
    }
    if (empty($attachments) && !$send_without_tickets) {
      return;
    }
  }

  // Set the sender name and from address.
  if (empty($from_mail)) {
    $from = NULL;
  }
  else {
    $from = array(
      'name' => $from_name,
      'mail' => $from_mail,
    );
    // Create an address string.
    module_load_include('inc', 'mimemail', 'mimemail');
    $from = mimemail_address($from);
  }

  // Set up all the other parameters for drupal_mail().
  $key = 'commerce_event_ticket_pdf_mail';
  $language = language_default();
  $params = array(
    'context' => array(
      'subject' => $subject,
      'body' => $body,
    ),
    'reply-to' => isset($params['reply_to']) ? $params['reply_to'] : NULL,
    'plaintext' => $plaintext,
    'attachments' => $attachments,
  );

  drupal_mail('commerce_event_ticket_pdf', $key, $to, $language, $params, $from);
}
