<?php
/**
 * @file
 * Resource callbacks for the Commerce Event Ticket Services module.
 */

/**
 * Retrieve a list of nodes that have event tickets.
 */
function cet_services_list_event_ticket_nodes($offset = 0, $limit = 50) {
  drupal_add_http_header('Access-Control-Allow-Origin', '*');

  if ($limit > 100) {
    $limit = 100;
  }

  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node');
  $query->propertyCondition('status', NODE_PUBLISHED);
  $query->range($offset, $limit);

  // Limit the list of nodes to those with tickets.
  // @todo make this more efficient
  $nids = db_query('SELECT DISTINCT nid FROM {commerce_event_ticket}')->fetchCol();
  if ($nids) {
    $query->propertyCondition('nid', $nids, 'IN');
  }

  // If date validation is configured, then limit the list of events to those
  // either in the future or starting less than 24 hours ago.
  $date_field_names = variable_get('commerce_event_ticket_date_fields', array());
  foreach ($date_field_names as $field_name) {
    $query->fieldCondition($field_name, 'value', REQUEST_TIME - 86400, '>');
    $query->fieldOrderBy($field_name, 'value');
  }

  $result = $query->execute();
  if (!isset($result['node'])) {
    return array();
  }

  $nids = array_keys($result['node']);
  $nodes = node_load_multiple($nids);

  // Build an array of simplified node information.
  $output = array();
  foreach ($nodes as $nid => $node) {
    // Add the node ID and node title.
    $output[$nid] = array('nid' => $nid, 'title' => $node->title);
    // Add the event start and end dates, if possible.
    foreach ($date_field_names as $field_name) {
      if (!isset($node->$field_name)) {
        continue;
      }
      $dates = $node->$field_name;
      if (!isset($dates[$node->language][0])) {
        continue;
      }
      $date = $dates[$node->language][0];
      if (isset($date['value'])) {
        $output[$nid]['start_date'] = $date['value'];
      }
      if (isset($date['value2']) && $date['value2'] != $date['value']) {
        $output[$nid]['end_date'] = $date['value2'];
      }
    }
  }
  return $output;
}

/**
 * Retrieve a list of tickets for a product display node.
 */
function cet_services_retrieve_tickets_per_node($nid, $offset = 0, $limit = 50, $changed_since = 0) {
  drupal_add_http_header('Access-Control-Allow-Origin', '*');

  if ($limit > 100) {
    $limit = 100;
  }

  // Find the ticket IDs.
  $ticket_ids = commerce_event_ticket_find_by_property('nid', $nid, '=', $offset, $limit, $changed_since);
  if (!$ticket_ids) {
    return array();
  }

  $tickets = commerce_event_ticket_load_multiple($ticket_ids);
  $view_all = user_access('view all commerce event tickets');

  $info = array();
  foreach ($tickets as $ticket) {
    // If the user doesn't have access to view all tickets, then only return
    // those whose products the user is allowed to update.
    if (!$view_all) {
      $product = commerce_product_load($ticket->product_id);
      if (!commerce_product_access('update', $product)) {
        continue;
      }
    }
    $valid = commerce_event_ticket_is_valid($ticket);
    $reason = commerce_event_ticket_explain_validity($ticket);
    $info[$ticket->ticket_id] = array(
      'valid' => $valid,
      'reason' => $valid ? NULL : $reason,
      'barcode_token' => $ticket->barcode_token,
      'ticket_id' => $ticket->ticket_id,
      'used' => (bool) $ticket->used,
      'position' => t('@number of @count', array(
        '@number' => $ticket->number,
        '@count' => $ticket->quantity,
      )),
      'created' => date('c', $ticket->created),
      'changed' => date('c', $ticket->changed),
    );
  }

  return $info;
}

/**
 * Validate callback to mark a ticket as used.
 */
function cet_services_validate_ticket($barcode, $log = NULL) {
  $ticket = commerce_event_ticket_load_by_barcode($barcode);
  if (!$ticket) {
    return services_error(t('Ticket not found'), 404);
  }
  $output = array();
  $log = empty($log) ? t('Validated via service.') : $log;
  $result = commerce_event_ticket_validate($ticket, TRUE, TRUE, $log);
  if ($result === TRUE) {
    $output['validated'] = TRUE;
  }
  else {
    drupal_add_http_header('Status', '400 ' . t('Ticket not validated'));
    $output['validated'] = FALSE;
    $output['reason'] = $result;
  }
  return $output;
}

/**
 * Validate callback to mark multiple tickets as used.
 */
function cet_services_validate_multiple_tickets($barcodes, $log = NULL) {
  drupal_add_http_header('Access-Control-Allow-Origin', '*');
  if (count($barcodes) > 100) {
    return services_error(t('Maximum number of tickets exceeded'), 400);
  }
  elseif (count($barcodes) == 0) {
    return services_error(t('No tickets specified'), 400);
  }
  $ticket_ids = db_query('SELECT barcode_token, ticket_id FROM {commerce_event_ticket} WHERE barcode_token IN (:barcodes)', array(
    ':barcodes' => $barcodes,
  ))->fetchAllKeyed();
  $tickets = commerce_event_ticket_load_multiple($ticket_ids);
  $output = array();
  $log = empty($log) ? t('Validated via service.') : $log;
  foreach ($barcodes as $barcode) {
    $output[$barcode] = array();
    if (!isset($ticket_ids[$barcode]) || !isset($tickets[$ticket_ids[$barcode]])) {
      $output[$barcode]['found'] = FALSE;
      $output[$barcode]['validated'] = FALSE;
      continue;
    }
    $ticket = $tickets[$ticket_ids[$barcode]];
    $output[$barcode]['found'] = TRUE;
    $result = commerce_event_ticket_validate($ticket, TRUE, TRUE, $log);
    if ($result === TRUE) {
      $output[$barcode]['validated'] = TRUE;
    }
    else {
      $output[$barcode]['validated'] = FALSE;
      $output[$barcode]['reason'] = $result;
    }
  }
  return $output;
}

/**
 * Retrieve callback to get ticket information based on a barcode.
 */
function cet_services_retrieve_ticket($barcode) {
  $ticket = commerce_event_ticket_load_by_barcode($barcode);
  if (!$ticket) {
    return services_error(t('Ticket not found'), 404);
  }
  $valid = commerce_event_ticket_is_valid($ticket);
  if ($valid) {
    $reason = NULL;
  }
  else {
    $reason = commerce_event_ticket_explain_validity($ticket);
  }
  $output = array(
    'valid' => commerce_event_ticket_is_valid($ticket),
    'reason' => $reason,
    'barcode_token' => $ticket->barcode_token,
    'ticket_id' => $ticket->ticket_id,
    'used' => (bool) $ticket->used,
    'position' => t('@number of @count', array(
      '@number' => $ticket->number,
      '@count' => $ticket->quantity,
    )),
    'created' => date('c', $ticket->created),
    'changed' => date('c', $ticket->changed),
  );
  $access_all_info = user_access('view all info for a commerce event ticket');
  $order = commerce_order_load($ticket->order_id);
  if ($order && ($access_all_info || commerce_order_access('view', $order))) {
    $output['order'] = array(
      'order_id' => $order->order_id,
      'order_number' => $order->order_number,
      'status' => $order->status,
      'status_title' => commerce_order_status_get_title($order->status),
      'created' => date('c', $order->created),
      'changed' => date('c', $order->changed),
    );
    $owner = user_load($order->uid);
    if ($owner && ($access_all_info || user_view_access($owner))) {
      $output['order']['owner'] = array(
        'uid' => $order->uid,
        'name' => $owner->name,
        'mail' => $owner->mail,
        'username' => format_username($owner),
        'status' => $owner->status,
      );
    }
  }
  $line_item = commerce_line_item_load($ticket->line_item_id);
  if ($line_item && ($access_all_info || commerce_line_item_access('view', $line_item))) {
    $line_item_wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
    $total_price = commerce_line_items_total(array($line_item_wrapper));
    $output['line_item'] = array(
      'line_item_id' => $line_item->line_item_id,
      'type' => $line_item->type,
      'quantity' => $line_item->quantity,
      'unit_price' => commerce_event_ticket_get_unit_price($ticket, $line_item_wrapper),
      'total_price' => commerce_currency_format($total_price['amount'], $total_price['currency_code']),
      'created' => date('c', $line_item->created),
      'changed' => date('c', $line_item->changed),
    );
  }
  $product = commerce_product_load($ticket->product_id);
  if ($product && ($access_all_info || commerce_product_access('view', $product))) {
    $output['product'] = array(
      'product_id' => $ticket->product_id,
      'title' => $product->title,
      'type' => $product->type,
      'status' => $product->status,
      'revision_id' => $product->revision_id,
      'created' => date('c', $product->created),
      'changed' => date('c', $product->changed),
    );
  }
  if ($ticket->nid) {
    $node = node_load($ticket->nid);
  }
  if (!empty($node) && ($access_all_info || node_access('view', $node))) {
    $uri = node_uri($node);
    $path = $uri['path'];
    $output['node'] = array(
      'nid' => $node->nid,
      'title' => $node->title,
      'type' => $node->type,
      'status' => $node->status,
      'vid' => $node->vid,
      'created' => date('c', $node->created),
      'changed' => date('c', $node->changed),
      'path_raw' => $GLOBALS['base_path'] . $path,
      'path' => url($path),
    );
  }
  return $output;
}
